package xyz.stachel.zombiesutils.config;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.JsonParser;
import net.fabricmc.loader.api.FabricLoader;
import xyz.stachel.zombiesutils.ZombiesUtils;

import java.lang.reflect.Modifier;
import java.nio.file.Files;
import java.nio.file.Path;

public final class Config {
    private static final Gson GSON = new GsonBuilder()
            .registerTypeAdapter(IntConfig.class, new IntConfig.Serializer())
            .registerTypeAdapter(DoubleConfig.class, new DoubleConfig.Serializer())
            .excludeFieldsWithModifiers(Modifier.TRANSIENT)
            .setPrettyPrinting()
            .create();
    private static final Path PATH = FabricLoader.getInstance().getConfigDir().resolve(String.format("%s.json", ZombiesUtils.MOD_ID));

    public static void readFile() {
        if (Files.exists(PATH)) {
            try (var reader = Files.newBufferedReader(PATH)) {
                var jsonElement = JsonParser.parseReader(reader);

                for (var field : Configs.class.getDeclaredFields()) {
                    var value = jsonElement.getAsJsonObject().get(field.getName());

                    if (value != null) {
                        var fieldType = field.getType();

                        if (IntConfig.class.isAssignableFrom(fieldType)) {
                            var intValue = value.getAsInt();
                            ((IntConfig) field.get(null)).setValue(intValue);
                        }

                        if (DoubleConfig.class.isAssignableFrom(fieldType)) {
                            var doubleValue = value.getAsDouble();
                            ((DoubleConfig) field.get(null)).setValue(doubleValue);
                        }
                    }
                }
            } catch (Exception exception) {
                ZombiesUtils.LOGGER.error("Failed to load configs at '{}'. Using default values.", PATH, exception);
            }
        } else {
            writeFile();
        }
    }

    public static void writeFile() {
        try (var writer = Files.newBufferedWriter(PATH)) {
            GSON.toJson(new Configs(), writer);
        } catch (Exception exception) {
            ZombiesUtils.LOGGER.error("Failed to save configs to '{}'.", PATH, exception);
        }
    }
}