package xyz.stachel.zombiesutils.game;

import net.minecraft.client.MinecraftClient;
import net.minecraft.text.Style;
import net.minecraft.text.Text;
import net.minecraft.util.Formatting;
import org.apache.commons.lang3.exception.ExceptionUtils;
import org.jetbrains.annotations.NotNull;
import xyz.stachel.zombiesutils.ZombiesUtils;
import xyz.stachel.zombiesutils.game.recorder.Category;
import xyz.stachel.zombiesutils.game.recorder.RecordMessageSender;
import xyz.stachel.zombiesutils.game.recorder.files.CategoryFile;

class Game extends Timer {
  private final boolean joinedRoundOne;
  private int round;
  private final GameFile gameFile;
  private final boolean roundOneRecorded;
  private Category category;
  public final GameMode mode;
  private boolean escaping;

  public Game(@NotNull final GameFile gameFile, @NotNull final GameMode mode) {
    super();
    this.gameFile = gameFile;
    this.round = 1;
    this.joinedRoundOne = true;
    this.mode = mode;
    this.roundOneRecorded = true;
  }

  public Game(@NotNull final GameFile gameFile, @NotNull final GameMode mode, final int round) {
    super();
    this.gameFile = gameFile;
    this.round = round;
    this.joinedRoundOne = round == 1;
    this.mode = mode;
    this.roundOneRecorded = round == 1;
  }

  public void setCategory(Category category) {
    this.category = category;
  }

  public void pass(int round) {
    if ((round == 0) || (this.round == round + 1) || (this.roundTime() < 100)) {
      ZombiesUtils.LOGGER.debug("SPLIT CANCELLED");
      return;
    }
    try {
       this.record();
    } catch (Exception e) {
      ZombiesUtils.LOGGER.error(ExceptionUtils.getStackTrace(e));
      MinecraftClient.getInstance().player.sendMessage(Text.literal("Error recording splits").setStyle(Style.EMPTY.withColor(Formatting.RED)), false);
    }
    this.split();
    this.round = round + 1;
  }

  public long split(final int round) {
    if (round == 1) {

    }
    return super.split();
  }

  public void helicopter() {
    if (!this.mode.isMap(GameMode.Map.PRISON)) {
      MinecraftClient.getInstance().player.sendMessage(Text.literal("You are not imprisoned. Yet.").setStyle(Style.EMPTY.withColor(Formatting.RED)), false);
      ZombiesUtils.LOGGER.error(Thread.currentThread().getStackTrace().toString());
      return;
    }
    this.escaping = true;
    this.pass(30);
  }

  private void record() {
    this.compareSegment();
    if (this.roundOneRecorded) this.compareBest();
    this.gameFile.setSegment(this.round, (int) this.roundTime());
  }

  public void compareSegment() throws IndexOutOfBoundsException {
    if (this.escaping) return;
    final CategoryFile categoryFile = this.category.getByGameMode(this.mode);
    final short bestSegment = categoryFile.getBestSegment(round);
    final int roundTime = (int) this.roundTime();

    if (bestSegment == (short) 0) categoryFile.setBestSegment(round, roundTime);
    else if (roundTime < bestSegment) categoryFile.setBestSegment(round, roundTime);
    final RecordMessageSender recordMessageSender = new RecordMessageSender(this.category.getName(), round, roundTime, bestSegment);
    recordMessageSender.roundSplit();
    recordMessageSender.sendRecordMessage();
  }

  public void compareBest() throws IndexOutOfBoundsException {
    final CategoryFile categoryFile = this.category.getByGameMode(this.mode);
    final int round = this.escaping ? 31 : this.round;
    final int personalBest = categoryFile.getPersonalBest(round);
    final int gameTime = (int) this.gameTime();

    if (personalBest == 0) categoryFile.setPersonalBest(round, gameTime);
    else if (gameTime < personalBest) categoryFile.setPersonalBest(round, gameTime);
    final RecordMessageSender recordMessageSender = new RecordMessageSender(category.getName(), round, gameTime, personalBest);
    if (!escaping) recordMessageSender.gameSplit();
    else recordMessageSender.helicopterSplit();
    recordMessageSender.sendRecordMessage();
  }
}
